# -*- coding: utf-8 -*-

"""Top-level package for iex-api."""

__author__ = "Volpyx"
__email__ = "thomas.kluiters@gmail.com"
# Do not edit this string manually, always use bumpversion
# Details in CONTRIBUTING.md
__version__ = "0.0.9"


def get_module_version():
    return __version__
